@@ -1,5 +1,5 @@
 // elaboration functions
-// Copyright (C) 2005, 2006 Red Hat Inc.
+// Copyright (C) 2005-2007 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -41,6 +41,7 @@ derived_probe::derived_probe (probe *p):
     {
       this->locations = p->locations;  
       this->tok = p->tok;
+      this->privileged = p->privileged;
       this->body = deep_copy_visitor::deep_copy(p->body);
     }
 }
@@ -55,6 +56,7 @@ derived_probe::derived_probe (probe *p, probe_point *l):
   if (p)
     {
       this->tok = p->tok;
+      this->privileged = p->privileged;
       this->body = deep_copy_visitor::deep_copy(p->body);
     }
 }
@@ -326,6 +328,15 @@ match_node::find_and_build (systemtap_session& s,
 }
 
 
+void
+match_node::build_no_more (systemtap_session& s)
+{
+  for (sub_map_iterator_t i = sub.begin(); i != sub.end(); i++)
+    i->second->build_no_more (s);
+  if (end) end->build_no_more (s);
+}
+
+
 // ------------------------------------------------------------------------
 // Alias probes
 // ------------------------------------------------------------------------
@@ -340,7 +351,7 @@ struct alias_derived_probe: public derived_probe
   // types, and are not themselves emitted or listed in
   // systemtap_session.probes
 
-  void join_group (systemtap_session& s) { upchuck (); }
+  void join_group (systemtap_session&) { upchuck (); }
 };
 
 
@@ -357,7 +368,7 @@ alias_expansion_builder
   virtual void build(systemtap_session & sess,
 		     probe * use, 
 		     probe_point * location,
-		     std::map<std::string, literal *> const & parameters,
+		     std::map<std::string, literal *> const &,
 		     vector<derived_probe *> & finished_results)
   {
     // We're going to build a new probe and wrap it up in an
@@ -914,6 +925,10 @@ semantic_pass_symbols (systemtap_session& s)
             }
         }
     }
+
+  // Inform all derived_probe builders that we're done with
+  // all resolution, so it's time to release caches.
+  s.pattern_root->build_no_more (s);
   
   return s.num_errors(); // all those print_error calls
 }
@@ -960,9 +975,7 @@ systemtap_session::systemtap_session ():
   mark_derived_probes(0), 
   hrtimer_derived_probes(0), 
   perfmon_derived_probes(0), 
-  op (0), up (0),
-  kprobes_text_initialized (false),
-  kprobes_text_start (0), kprobes_text_end (0)
+  op (0), up (0)
 {
 }
 
@@ -1809,15 +1822,17 @@ semantic_pass_types (systemtap_session& s)
         }
       
       if (ti.num_newly_resolved == 0) // converged
-        if (ti.num_still_unresolved == 0)
-          break; // successfully
-        else if (! ti.assert_resolvability)
-          ti.assert_resolvability = true; // last pass, with error msgs
-        else
-          { // unsuccessful conclusion
-            rc ++;
-            break;
-          }
+        {
+          if (ti.num_still_unresolved == 0)
+            break; // successfully
+          else if (! ti.assert_resolvability)
+            ti.assert_resolvability = true; // last pass, with error msgs
+          else
+            { // unsuccessful conclusion
+              rc ++;
+              break;
+            }
+        }
     }
   
   return rc + s.num_errors();
@@ -2301,7 +2316,7 @@ typeresolution_info::visit_block (block* e)
 
 
 void
-typeresolution_info::visit_embeddedcode (embeddedcode* e)
+typeresolution_info::visit_embeddedcode (embeddedcode*)
 {
 }
 
@@ -2405,7 +2420,7 @@ typeresolution_info::visit_foreach_loop (foreach_loop* e)
 
 
 void
-typeresolution_info::visit_null_statement (null_statement* e)
+typeresolution_info::visit_null_statement (null_statement*)
 {
 }
 
@@ -2451,19 +2466,19 @@ typeresolution_info::visit_delete_statement (delete_statement* e)
 
 
 void
-typeresolution_info::visit_next_statement (next_statement* s)
+typeresolution_info::visit_next_statement (next_statement*)
 {
 }
 
 
 void
-typeresolution_info::visit_break_statement (break_statement* s)
+typeresolution_info::visit_break_statement (break_statement*)
 {
 }
 
 
 void
-typeresolution_info::visit_continue_statement (continue_statement* s)
+typeresolution_info::visit_continue_statement (continue_statement*)
 {
 }
 
@@ -2716,7 +2731,7 @@ typeresolution_info::mismatch (const token* tok, exp_type t1, exp_type t2)
 
 
 void
-typeresolution_info::resolved (const token* tok, exp_type t)
+typeresolution_info::resolved (const token*, exp_type)
 {
   num_newly_resolved ++;
   // cerr << "resolved " << *e->tok << " type " << t << endl;
