@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005, 2006 Red Hat Inc.
+// Copyright (C) 2005-2007 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -9,7 +9,6 @@
 #ifndef ELABORATE_H
 #define ELABORATE_H
 
-#include "session.h"
 #include "staptree.h"
 #include "parse.h"
 #include <string>
@@ -106,6 +105,7 @@ struct typeresolution_info: public visitor
 // provider may transform it.
 
 class translator_output;
+class derived_probe_group;
 
 struct derived_probe: public probe
 {
@@ -113,39 +113,53 @@ struct derived_probe: public probe
   derived_probe (probe* b, probe_point* l);
   probe* base; // the original parsed probe
   virtual probe* basest () { return base->basest(); }
-
   virtual ~derived_probe () {}
+  virtual void join_group (systemtap_session& s) = 0;
+  virtual probe_point* sole_location ();
 
-  virtual void emit_registrations (translator_output* o) = 0;
-  // (from within module_init):
-  // rc = ..... register_or_whatever (ENTRYFN);
-
-  virtual void emit_deregistrations (translator_output* o) = 0;
-  // (from within module_exit):
-  // (void) ..... unregister_or_whatever (ENTRYFN);
-
-  virtual void emit_probe_entries (translator_output* o) = 0;
-  // ... for all probe-points:
-  // ELABORATE_SPECIFIC_SIGNATURE ENTRYFN {
-  //   /* allocate context - probe_prologue */
-  //   /* copy parameters, initial state into context */
-  //   probe_NUMBER (context);
-  //   /* deallocate context - probe_epilogue */
-  // }
-
-  virtual void emit_probe_context_vars (translator_output* o) {}
+  virtual void emit_probe_context_vars (translator_output*) {}
   // From within unparser::emit_common_header, add any extra variables
   // to this probe's context locals.
 
-protected:
-  void emit_probe_prologue (translator_output* o, const std::string&);
-  void emit_probe_epilogue (translator_output* o);
-
 public:
   static void emit_common_header (translator_output* o);
   // from c_unparser::emit_common_header
+  // XXX: probably can move this stuff to a probe_group::emit_module_decls
 };
 
+// ------------------------------------------------------------------------
+
+struct unparser;
+
+// Various derived classes derived_probe_group manage the
+// registration/invocation/unregistration of sibling probes.
+struct derived_probe_group
+{
+  virtual ~derived_probe_group () {}
+
+  virtual void emit_module_decls (systemtap_session& s) = 0;
+  // The _decls-generated code may assume that declarations such as
+  // the context, embedded-C code, function and probe handler bodies
+  // are all already generated.  That is, _decls is called near the
+  // end of the code generation process.  It should minimize the
+  // number of separate variables (and to a lesser extent, their
+  // size).
+
+  virtual void emit_module_init (systemtap_session& s) = 0;
+  // The _init-generated code may assume that it is called only once.
+  // If that code fails at run time, it must set rc=1 and roll back
+  // any partial initializations, for its _exit friend will NOT be
+  // invoked.  The generated code may use pre-declared "int i, j;".
+
+  virtual void emit_module_exit (systemtap_session& s) = 0;
+  // The _exit-generated code may assume that it is executed exactly
+  // zero times (if the _init-generated code failed) or once.  (_exit
+  // itself may be called a few times, to generate the code in a few
+  // different places in the probe module.)
+  // The generated code may use pre-declared "int i, j;".
+};
+
+
 // ------------------------------------------------------------------------
 
 struct derived_probe_builder
@@ -156,6 +170,7 @@ struct derived_probe_builder
 		     std::map<std::string, literal*> const & parameters,
 		     std::vector<derived_probe*> & finished_results) = 0;
   virtual ~derived_probe_builder() {}
+  virtual void build_no_more (systemtap_session &) {}
 
   static bool get_param (std::map<std::string, literal*> const & parameters,
                          const std::string& key, std::string& value);
@@ -196,6 +211,7 @@ match_node
   void find_and_build (systemtap_session& s,
                        probe* p, probe_point *loc, unsigned pos,
                        std::vector<derived_probe *>& results);
+  void build_no_more (systemtap_session &s);
 
   match_node* bind(match_key const & k);
   match_node* bind(std::string const & k);
@@ -218,4 +234,7 @@ void derive_probes (systemtap_session& s,
 symbol * get_symbol_within_expression (expression *e);
 
 
+struct unparser;
+
+
 #endif // ELABORATE_H
