@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2006 Red Hat Inc.
+// Copyright (C) 2005-2007 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -46,7 +46,7 @@ version ()
     << "SystemTap translator/driver "
     << "(version " << VERSION << " built " << DATE << ")" << endl
     << "(Using " << dwfl_version (NULL) << " libraries.)" << endl
-    << "Copyright (C) 2005-2006 Red Hat, Inc. and others" << endl
+    << "Copyright (C) 2005-2007 Red Hat, Inc. and others" << endl
     << "This is free software; see the source for copying conditions." << endl;
 }
 
@@ -71,8 +71,9 @@ usage (systemtap_session& s, int exitcode)
     << "   -k         keep temporary directory" << endl
     << "   -u         unoptimized translation" << (s.unoptimized ? " [set]" : "") << endl
     << "   -g         guru mode" << (s.guru_mode ? " [set]" : "") << endl
-    << "   -b         bulk (relayfs) mode" << (s.bulk_mode ? " [set]" : "") << endl
-    << "   -M         Don't merge per-cpu files for bulk (relayfs) mode" << (s.merge ? "" : " [set]") << endl
+    << "   -P         prologue-searching for function probes" 
+    << (s.prologue_searching ? " [set]" : "") << endl
+    << "   -b         bulk (percpu file) mode" << (s.bulk_mode ? " [set]" : "") << endl
     << "   -s NUM     buffer size in megabytes, instead of "
     << s.buffer_size << endl
     << "   -p NUM     stop after pass NUM 1-5, instead of "
@@ -197,6 +198,13 @@ main (int argc, char * const argv [])
   s.guru_mode = false;
   s.bulk_mode = false;
   s.unoptimized = false;
+
+#ifdef ENABLE_PROLOGUES
+  s.prologue_searching = true;
+#else
+  s.prologue_searching = false;
+#endif
+
   s.buffer_size = 0;
   s.last_pass = 5;
   s.module_name = "stap_" + stringify(getpid());
@@ -256,7 +264,8 @@ main (int argc, char * const argv [])
 
   while (true)
     {
-      int grc = getopt (argc, argv, "hVMvtp:I:e:o:R:r:m:kgc:x:D:bs:u");
+      // NB: also see find_hash(), help(), switch stmt below, stap.1 man page
+      int grc = getopt (argc, argv, "hVMvtp:I:e:o:R:r:m:kgPc:x:D:bs:u");
       if (grc < 0)
         break;
       switch (grc)
@@ -328,6 +337,10 @@ main (int argc, char * const argv [])
           s.guru_mode = true;
           break;
 
+        case 'P':
+          s.prologue_searching = true;
+          break;
+
         case 'b':
           s.bulk_mode = true;
           break;
@@ -623,17 +636,15 @@ main (int argc, char * const argv [])
     {
       ostringstream o;
       unsigned saved_verbose;
-	  
-      // Make sure we're in verbose mode, so that printscript()
-      // will output function/probe bodies.
-      saved_verbose = s.verbose;
-      s.verbose = 3;
 
-      // Print script to 'o'
-      printscript(s, o);
-
-      // Restore original verbose mode setting.
-      s.verbose = saved_verbose;
+      {
+        // Make sure we're in verbose mode, so that printscript()
+        // will output function/probe bodies.
+        saved_verbose = s.verbose;
+        s.verbose = 3;
+        printscript(s, o);  // Print script to 'o'
+        s.verbose = saved_verbose;
+      }
 
       // Generate hash
       find_hash (s, o.str());
@@ -685,6 +696,10 @@ main (int argc, char * const argv [])
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
   rc = compile_pass (s);
+
+  if (rc == 0 && s.last_pass == 4)
+    cout << s.hash_path << endl;
+
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
 
@@ -704,9 +719,9 @@ main (int argc, char * const argv [])
       add_to_cache(s);
     }
 
-  // XXX: what to do if rc==0 && last_pass == 4?  dump .ko file to stdout?
   if (rc || s.last_pass == 4) goto cleanup;
 
+
   // PASS 5: RUN
 pass_5:
   times (& tms_before);
