@@ -23,7 +23,7 @@
 /* A flag indicate whether to store the trace
    data into local file/MySQL database */
 int into_file, into_db;
-
+int name_flag=1, id_flag=0, appname_flag=1;
 #ifdef HAS_MYSQL
 
 #define SQLSIZE 1024*1024
@@ -63,13 +63,28 @@ GTree *appNameTree;
 /* event table */
 event_desc *events_des[MAX_EVT_TYPES][MAX_GRPID][MAX_HOOKID]; 
 
+#define b2a_error(fmt, args...) do { fprintf(stderr, "Error %d@%s: "fmt, __LINE__, __func__, ## args);\
+                exit(-1); } while(0)
+
 void usage()
 {
 printf("Usage:\n\
   lket-b2a Options INFILE1 [INFILE2...]\n\
     Options:\n\
        -f     dump the trace data into a local file named \"lket.out\"\n\
-       -m     dump the trace data into MySQL\n");
+       -n     name_flag. name_flag set to 0 means not printing the event\n\
+              description string and 1 means printing. Only valid with -f\n\
+              option. name_flag is set to 1 by default.\n\
+       -i     id_flag. id_flag set to 0 means not printing event groupid and\n\
+              hookid and 1 means printing. Only valid with -f option. id_flag\n\
+              is set to 0 by default.\n\
+       -a     appname_flag. appname_flag set to 0 means not printing process\n\
+              name and 1 means printing. Only valid with -f option. appname_flag\n\
+              is set to 1 by default.\n\
+       -m     dump the trace data into MySQL\n\
+   Example:\n\
+       lket-b2a -f -a 1 -i 1 -n 0 stpd_cpu*\n\
+       lket-b2a -m stpd_cpu*\n");
 }
 
 int main(int argc, char *argv[])
@@ -79,6 +94,7 @@ int main(int argc, char *argv[])
 	char	outfilename[MAX_STRINGLEN]={0};
 	int	i, j, total_infiles = 0;
 	long long min;
+	int retvalue = 0;
 
 	char database[18];
 	time_t timer;
@@ -89,7 +105,7 @@ int main(int argc, char *argv[])
         strftime(database, 18,  "DB%Y%m%d%H%M%S", tm);
 
         while (1) {
-                int c = getopt(argc, argv, "mf");
+                int c = getopt(argc, argv, "mfi:n:a:");
                 if (c < 0) // no more options
                         break;
                 switch (c) {
@@ -99,6 +115,31 @@ int main(int argc, char *argv[])
 		case 'f':
 			into_file = 1;
 			break;
+		case 'n':
+			name_flag = atoi(optarg);
+			if(name_flag!=0 && name_flag!=1) {
+				fprintf(stderr, "you must specify 0 or 1 for -n option\n");
+				usage();
+				exit(-1);
+			}
+			break;
+		case 'i':
+			id_flag = atoi(optarg);
+			if(id_flag!=0 && id_flag!=1) {
+				fprintf(stderr, "you must specify 0 or 1 for -i option\n");
+				usage();
+				exit(-1);
+			}
+			break;
+		case 'a':
+			appname_flag = atoi(optarg);
+			if(appname_flag!=0 && appname_flag!=1) {
+				fprintf(stderr, "you must specify 0 or 1 for -a option\n");
+				usage();
+				exit(-1);
+			}
+			break;
+
 		default:
 			printf("Error in options\n");
 			usage();
@@ -109,8 +150,7 @@ int main(int argc, char *argv[])
 
 #ifndef HAS_MYSQL
 	if(into_db)  {
-		fprintf(stderr, "-m option is not supported since lket-b2a is not compiled with mysql support, \n");
-		exit(-1);
+		b2a_error("-m option is not supported since lket-b2a is not compiled with mysql support\n");
 	}
 #endif
 	if(into_file==0 && into_db==0)  {
@@ -128,8 +168,7 @@ int main(int argc, char *argv[])
 	// open the input files and the output file
 	infps = (FILE **)malloc(total_infiles * sizeof(FILE *));
 	if(!infps) {
-		printf("Unable to malloc infps\n");
-		return 1;
+		b2a_error("Unable to malloc infps\n");
 	}
 
 	memset(infps, 0, total_infiles * sizeof(FILE *));
@@ -137,6 +176,7 @@ int main(int argc, char *argv[])
 		infps[i] = fopen(argv[optind++], "r");
 		if(infps[i] == NULL) {
 			printf("Unable to open %s\n", argv[optind-1]);
+			retvalue = -1;
 			goto failed;
 		}
 	}
@@ -144,9 +184,11 @@ int main(int argc, char *argv[])
 	if(into_file)  {
 		if(strnlen(outfilename, MAX_STRINGLEN) == 0)
 			strncpy(outfilename, DEFAULT_OUTFILE_NAME, MAX_STRINGLEN);
+	
 		outfp = fopen(outfilename, "w");
 		if(outfp == NULL) {
 			fprintf(stderr,"Unable to create %s\n", outfilename);
+			retvalue = -1;
 			goto failed;
 		}
 	}
@@ -156,25 +198,26 @@ int main(int argc, char *argv[])
 #ifdef HAS_MYSQL
 	if(into_db)  {
 		if(!mysql_init(&mysql))  {
-			fprintf(stderr, "Failed to Init MySQL: Error: %s\n",
+			b2a_error("Failed to Init MySQL: Error: %s\n",
 				mysql_error(&mysql));
 		}
+
 		if(!mysql_real_connect(&mysql, NULL, NULL, NULL, NULL, 0, NULL, 
 			CLIENT_MULTI_STATEMENTS))  {
-			fprintf(stderr, "Failed to connect to database: Error: %s\n",
+			b2a_error("Failed to connect to database: Error: %s\n",
 				mysql_error(&mysql));
 		}
 
 		snprintf(sql, 64,"create database %s", database);
 
 		if(mysql_query(&mysql, sql))  {
-			fprintf(stderr, "Failed create database %s, Error: %s\n",
+			b2a_error("Failed create database %s, Error: %s\n",
 				database, mysql_error(&mysql));
 		}
 
 		if(!mysql_real_connect(&mysql, NULL, NULL, NULL, database, 0, NULL, 
 			CLIENT_MULTI_STATEMENTS))  {
-			fprintf(stderr, "Failed to connect to database %s: Error: %s\n",
+			b2a_error("Failed to connect to database %s: Error: %s\n",
 				database, mysql_error(&mysql));
 		}
 	}
@@ -187,6 +230,7 @@ int main(int argc, char *argv[])
 	hdrs = malloc(total_infiles * sizeof(lket_pkt_header));
 	if(!hdrs) {
 		printf("Unable to malloc hdrs \n");
+		retvalue = -1;
 		goto failed;
 	}
 	memset(hdrs, 0, total_infiles * sizeof(lket_pkt_header));
@@ -226,7 +270,8 @@ int main(int argc, char *argv[])
 
 				if(HDR_GroupID(&hdrs[j])==_GROUP_PROCESS &&
 					(HDR_HookID(&hdrs[j])==_HOOKID_PROCESS_SNAPSHOT 
-					|| HDR_HookID(&hdrs[j])==_HOOKID_PROCESS_EXECVE))
+					|| HDR_HookID(&hdrs[j])==_HOOKID_PROCESS_EXECVE
+					|| HDR_HookID(&hdrs[j])==_HOOKID_PROCESS_FORK))
 				{
 					register_appname(j, infps[j], &hdrs[j]);
 				}
@@ -284,24 +329,23 @@ failed:
 
 	for(i=1; i<MAX_GRPID; i++)
 	for(j=1; j<MAX_HOOKID; j++) {
-		if(events_des[1][i][j] != NULL) {
+		if(events_des[_HOOKID_REGSYSEVT][i][j] != NULL) {
 #ifdef HAS_MYSQL
-			if(events_des[1][i][j]->flag == 0 && into_db)  {
+			if(events_des[_HOOKID_REGSYSEVT][i][j]->flag == 0 && into_db)  {
 				snprintf(sql, 256, "drop table %d_%d",i,j);
 				if(mysql_query(&mysql,sql))  {
-					fprintf(stderr, "Failed to exec sql: %s, Error: %s\n",
+					b2a_error("Failed to exec sql: %s, Error: %s\n",
 						sql, mysql_error(&mysql));
-					exit(-1);
 				}
 				snprintf(sql, 256, "delete from table_desc where table_name='%d_%d'",i,j);
 				if(mysql_query(&mysql,sql))  {
-					fprintf(stderr, "Failed to exec sql: %s, Error: %s\n",
+					b2a_error("Failed to exec sql: %s, Error: %s\n",
 						sql, mysql_error(&mysql));
-					exit(-1);
 				}
 			}
-			if(events_des[1][i][j]->entrytime) /* destroy entrytime tree */
-				g_tree_destroy(events_des[1][i][j]->entrytime);
+			/* destroy entrytime tree */
+			if(events_des[_HOOKID_REGSYSEVT][i][j]->entrytime)
+				g_tree_destroy(events_des[_HOOKID_REGSYSEVT][i][j]->entrytime);
 #endif
 		}
 	}
@@ -314,14 +358,14 @@ failed:
 	if (appNameTree)
 		g_tree_destroy(appNameTree);
 
-	return 0;
+	return retvalue;
 }
 
 /* register newly found process name for addevent.process.snapshot
   and addevent.process.execve */
 void register_appname(int i, FILE *fp, lket_pkt_header *phdr)
 {
-	int pid;
+	int pid, tid, ppid;
 	char *appname=NULL;
 	int count;
 	int len;
@@ -335,11 +379,9 @@ void register_appname(int i, FILE *fp, lket_pkt_header *phdr)
 
 	if(into_db)  {
 		if(flag==0)  {
-			if(mysql_query(&mysql, "create table appNameMap ( \
-				pid INT, pname varchar(20))"))  {
-				fprintf(stderr, "Failed to create appNameMap table, Error: %s\n",
+			if(mysql_query(&mysql, "create table appNameMap ( pid INT, pname varchar(20))"))  {
+				b2a_error("Failed to create appNameMap table, Error: %s\n",
 					mysql_error(&mysql));
-				exit(-1);
 			}
 		}
 		flag=1;
@@ -348,10 +390,10 @@ void register_appname(int i, FILE *fp, lket_pkt_header *phdr)
 	appname = (char *)malloc(512);
 	location = ftell(fp);
 
-	if(HDR_HookID(phdr) ==1 )  {  /* process_snapshot */
-		fseek(fp, 4, SEEK_CUR); /* skip tid */
+	if(HDR_HookID(phdr) == _HOOKID_PROCESS_SNAPSHOT )  {  /* process_snapshot */
+		fread(&tid, 1, 4, fp); /* read tid */
 		fread(&pid, 1, 4, fp); /* read pid */
-		fseek(fp, 4, SEEK_CUR); /* skip ppid */
+		fread(&ppid, 1, 4, fp); /* read ppid */
 		c = fgetc_unlocked(fp);
 		len+=13;
 		while (c && len < 1024) {
@@ -360,21 +402,10 @@ void register_appname(int i, FILE *fp, lket_pkt_header *phdr)
 			++len;
 		}
 		appname[count]='\0';
-#ifdef HAS_MYSQL
-		if(into_db)  {
-			snprintf(sql, 256, "insert into appNameMap values \
-				( %d, \"%s\")", pid, appname);
-			if(mysql_query(&mysql,sql))  {
-				fprintf(stderr, "Failed to exec sql: %s, Error: %s\n",
-					sql, mysql_error(&mysql));
-			exit(-1);
-			}
-		}
-#endif
-		//fseek(fp, 0-len, SEEK_CUR);
-	} else if (HDR_HookID(phdr) == 2)  { /* process.execve */
+	} else if (HDR_HookID(phdr) == _HOOKID_PROCESS_EXECVE)  { /* process.execve */
+		fread(&tid, 1, 4, fp); /* read tid */
 		fread(&pid, 1, 4, fp); /* read pid */
-
+		fread(&ppid, 1, 4, fp); /* read ppid */
 		c = fgetc_unlocked(fp);
 		len+=5;
 		while (c && len < 1024) {
@@ -383,23 +414,32 @@ void register_appname(int i, FILE *fp, lket_pkt_header *phdr)
 			++len;
 		}
 		appname[count]='\0';
-#ifdef HAS_MYSQL
-		if(into_db)  {
-			snprintf(sql, 256,"insert into appNameMap values \
-				( %d, \"%s\")", pid, appname);
-			if(mysql_query(&mysql,sql))  {
-				fprintf(stderr, "Failed to exec SQL: %s, Error: %s\n",
-					sql, mysql_error(&mysql));
-			exit(-1);
-			}
-		}
-#endif
-		//fseek(fp, 0-len, SEEK_CUR);
-	} else  {
+	} else  if (HDR_HookID(phdr) == _HOOKID_PROCESS_FORK) {
+		fread(&tid, 1, 4, fp); /* read tid */
+		fread(&pid, 1, 4, fp); /* read pid */
+		fread(&ppid, 1, 4, fp); /* read ppid */
+
+		char *appname_ptr;
+		appname_ptr = (char *)(g_tree_lookup(appNameTree,(gconstpointer)((long)ppid)));
+		if(appname_ptr == NULL)
+			strncpy(appname, "N/A", 4);
+		else
+			strncpy(appname, appname_ptr, 256);
+
+	} else {
 		free(appname);
 		return;
 	}
 	fseek(fp, location, SEEK_SET);
+#ifdef HAS_MYSQL
+		if(into_db)  {
+			snprintf(sql, 256,"insert into appNameMap values ( %d, \"%s\")", pid, appname);
+			if(mysql_query(&mysql,sql))  {
+				b2a_error("Failed to exec SQL: %s, Error: %s\n",
+					sql, mysql_error(&mysql));
+			}
+		}
+#endif
 	g_tree_insert(appNameTree, (gpointer)((long)pid), (gpointer)appname);
 }
 
@@ -436,7 +476,7 @@ void find_init_header(FILE **infps, const int total_infiles)
 	char  timing_methods_str[128];
 
 	if(total_infiles <= 0 )
-		return;
+		b2a_error("total_infiles <= 0\n");
 	j = total_infiles;
 	for(i=0; i<total_infiles; i++) {
 		if(fread(&magic, 1, sizeof(magic), infps[i]) < sizeof(magic))
@@ -513,21 +553,16 @@ void find_init_header(FILE **infps, const int total_infiles)
 
 #ifdef HAS_MYSQL
 	if(into_db)  {
-		if(mysql_query(&mysql, "create table trace_header \
-			( Major_Ver TINYINT, Minor_Ver TINYINT, \
-			Big_Endian TINYINT, Timing_Method varchar(20), \
-			Bits_Width TINYINT)" )) {
-			fprintf(stderr, "Failed to create trace_header table, Error: %s\n",
+		if(mysql_query(&mysql, "create table trace_header ( Major_Ver TINYINT, Minor_Ver TINYINT, Big_Endian TINYINT, Timing_Method varchar(20), Bits_Width TINYINT)" )) {
+			b2a_error("Failed to create trace_header table, Error: %s\n",
 				mysql_error(&mysql));
-			exit(-1);
 		}
 		snprintf(sql, 256, "insert into trace_header value ( %d, %d, %d, \"%s\", %d )",
 			ver_major, ver_minor, big_endian, timing_methods_str, bits_width);	
 
 		if(mysql_query(&mysql, sql)) {
-			fprintf(stderr, "Failed exec SQL: \n %s \n, Error: %s\n",
-				sql, mysql_error(&mysql));
-			exit(-1);
+			b2a_error("Failed exec SQL %d: \n %s \n, Error: %s\n",
+				__LINE__, sql, mysql_error(&mysql));
 		}
 	}
 #endif
@@ -541,27 +576,29 @@ void find_init_header(FILE **infps, const int total_infiles)
  */
 int get_pkt_header(FILE *fp, lket_pkt_header *phdr)
 {
-	if(fread(phdr, 1, sizeof(lket_pkt_header), fp) < sizeof(lket_pkt_header))
-		goto bad;
+	size_t size;
+	if(feof(fp)) return 0;
+	if((size = fread(phdr, 1, sizeof(lket_pkt_header), fp)) < sizeof(lket_pkt_header))  {
+		if(feof(fp))  {
+			bzero(phdr, sizeof(lket_pkt_header));
+			return 0;
+		} else 
+			b2a_error("fread read %u bytes than expected %u, feof:%d\n", (unsigned) size, (unsigned) sizeof(lket_pkt_header), feof(fp));
+	}
 
 	phdr->sys_size -= sizeof(lket_pkt_header)-sizeof(phdr->total_size)-sizeof(phdr->sys_size);
 	phdr->total_size -= sizeof(lket_pkt_header)-sizeof(phdr->total_size)-sizeof(phdr->sys_size);
-
 	return 0;
-
-bad:
-	memset(phdr, 0, sizeof(lket_pkt_header));
-	return -1;
 }	
 
 void print_pkt_header(lket_pkt_header *phdr)
 {
 	long long usecs;
 	int sec, usec;
-	int grpid, hookid, pid;
+	int grpid, hookid, pid, tid, ppid;
 
 	if(!phdr)
-		return;
+		b2a_error("phdr is NULL\n");
 
 	if(timing_method == TIMING_GETCYCLES)
 		usecs = (phdr->microsecond - cpufreq[HDR_CpuID(phdr)].last_cycles)
@@ -577,39 +614,52 @@ void print_pkt_header(lket_pkt_header *phdr)
 	grpid = HDR_GroupID(phdr);	
 	hookid = HDR_HookID(phdr);
 	pid = HDR_PID(phdr);
+	tid = HDR_TID(phdr);
+	ppid = HDR_PPID(phdr);
 
-	if(into_file)
-		fprintf(outfp, "\n%d.%d APPNAME: %s PID:%d CPU:%d HOOKGRP:%d HOOKID:%d ",
-			sec, usec,
-			(char *)(g_tree_lookup(appNameTree, (gconstpointer)((long)pid))),
-			pid, HDR_CpuID(phdr), grpid, hookid);
+	if(into_file) {
+		fprintf(outfp, "\n%d.%d CPU:%d TID:%d, PID:%d, PPID:%d, ", sec, usec, 
+			HDR_CpuID(phdr), tid, pid, ppid);
+		if(appname_flag==1)  {
+			char *appname_ptr;
+			appname_ptr = (char *)(g_tree_lookup(appNameTree,(gconstpointer)((long)pid)));
+			if(appname_ptr == NULL)  
+				fprintf(outfp, "APPNAME:N/A ");
+			else
+				fprintf(outfp, "APPNAME:%s ", appname_ptr);
+		}
+		if(name_flag==1)
+			fprintf(outfp, "EVT_NAME:%s ", events_des[_HOOKID_REGSYSEVT][grpid][hookid]->description);
+		if(id_flag==1)
+			fprintf(outfp, "HOOKGRP:%d HOOKID:%d ", grpid, hookid);
+	}
 
 #ifdef HAS_MYSQL
 	if(into_db)  {
 		if(!(hookid%2)) { // return type event
 			long long *entrytime;
 			long long entryusecs;
-			entrytime = g_tree_lookup(events_des[1][grpid][hookid-1]->entrytime,
-                                        (gconstpointer)((long)pid));
+			entrytime = g_tree_lookup(events_des[_HOOKID_REGSYSEVT][grpid][hookid-1]->entrytime,
+                                        (gconstpointer)((long)tid));
 			if(entrytime==NULL)  // key not found
 				entryusecs = 0;
 			else
 				entryusecs = *entrytime;
-			snprintf(sql_col, 64, "groupid, hookid, usec, process_id, \
+			snprintf(sql_col, 128, "groupid, hookid, usec, thread_id, process_id, parentprocess_id, \
 				cpu_id, entry_usec,");
-			snprintf(sql_val, 256, "%d, %d, %lld, %d, %d, %lld,", grpid,
-				hookid, usecs, pid, HDR_CpuID(phdr), 
+			snprintf(sql_val, 256, "%d, %d, %lld, %d, %d, %d, %d, %lld,", grpid,
+				hookid, usecs, tid, pid, ppid, HDR_CpuID(phdr), 
 				entryusecs);
 		}  else  {
-			snprintf(sql_col, 64, "groupid, hookid, usec, process_id, cpu_id,");
-			snprintf(sql_val, 256, "%d, %d, %lld, %d, %d, ", grpid, 
-				hookid, usecs, pid, HDR_CpuID(phdr));
+			snprintf(sql_col, 128, "groupid, hookid, usec, thread_id, process_id, parentprocess_id, cpu_id,");
+			snprintf(sql_val, 256, "%d, %d, %lld, %d, %d, %d, %d, ", grpid, 
+				hookid, usecs, tid, pid, ppid, HDR_CpuID(phdr));
 		}
 		if(hookid%2) {
 			char *entrytime = malloc(sizeof(long long));
 			*((long long *)entrytime) = usecs;
-			g_tree_insert(events_des[1][grpid][hookid]->entrytime, 
-				(gpointer)((long)pid), (gpointer)entrytime);
+			g_tree_insert(events_des[_HOOKID_REGSYSEVT][grpid][hookid]->entrytime, 
+				(gpointer)((long)tid), (gpointer)entrytime);
 		}
 	}
 #endif
@@ -639,31 +689,37 @@ void register_evt_desc(FILE *infp, size_t size)
 	static int has_table = 0;
 #endif
 	int grpid, hookid;
+	int len = 0;
 	char *evt_body;
 	evt_body = malloc(size);
 	fread(evt_body, size, 1, infp);
 	grpid = *(int8_t *)evt_body;
         hookid  = *(int8_t *)(evt_body+1);
+	len = strlen(evt_body+2)+2;
+	if(!events_des[_HOOKID_REGSYSEVT][grpid][hookid])
+		events_des[_HOOKID_REGSYSEVT][grpid][hookid] = malloc(sizeof(event_desc));
+	events_des[_HOOKID_REGSYSEVT][grpid][hookid]->description = malloc(len);
 
-	if(!events_des[1][grpid][hookid])
-		events_des[1][grpid][hookid] = malloc(sizeof(event_desc));
+	strncpy(events_des[_HOOKID_REGSYSEVT][grpid][hookid]->description, evt_body+2, len);
 #ifdef HAS_MYSQL
-	events_des[1][grpid][hookid]->entrytime = g_tree_new_full(
+	events_des[_HOOKID_REGSYSEVT][grpid][hookid]->entrytime = g_tree_new_full(
 		compareFunc, NULL, NULL, destroyTreeData);
 	if(into_db)  {
 		if(!has_table)  {
-			snprintf(sql, 1024, "create table table_desc ( table_name \
-				varchar(6), table_desc varchar(32))");
+			snprintf(sql, 1024, "create table table_desc ( table_name varchar(6), table_desc varchar(32))");
+			if(mysql_query(&mysql, sql))  {
+				b2a_error("Failed exec SQL: \n %s \n, Error: %s\n",
+					sql, mysql_error(&mysql));
+			}
 			has_table = 1;
-		} else  {
-			snprintf(sql, 1024, "insert into table_desc ( table_name,\
-				table_desc) values ( \"%d_%d\", \"%s\")", grpid, hookid,
-				evt_body+2);
-		}
+		} 
+
+		snprintf(sql, 1024, "insert into table_desc ( table_name, table_desc) values ( \"%d_%d\", \"%s\")", grpid, hookid,
+			evt_body+2);
+
 		if(mysql_query(&mysql, sql))  {
-			fprintf(stderr, "Failed exec SQL: \n %s \n, Error: %s\n",
+			b2a_error("Failed exec SQL:\n %s \n, Error: %s\n",
 				sql, mysql_error(&mysql));
-			exit(-1);
 		}
 	}
 #endif
@@ -687,27 +743,21 @@ void register_events(int evt_type, FILE *infp, size_t size)
 	if(!events_des[evt_type][grpid][hookid])
 		events_des[evt_type][grpid][hookid] = malloc(sizeof(event_desc));
 	if(!events_des[evt_type][grpid][hookid])  {
-		fprintf(stderr, "error when malloc for event_des[%d][%d][%d]\n",
+		b2a_error("error when malloc for event_des[%d][%d][%d]\n",
 			evt_type, grpid, hookid);
 	}
 
 #ifdef HAS_MYSQL
 	if(into_db)  {
-		if(evt_type==1)  {  /* if sys event, create a table */
+		if(evt_type==_HOOKID_REGSYSEVT)  {  /* if sys event, create a table */
 			if(!(hookid%2))  {/* if this is a return type event, should record 
 					     the entry time of this event */
-				snprintf(sql, 1024, "create table %d_%d \
-					( groupid TINYINT, hookid TINYINT, usec BIGINT,\
-					 process_id INT, cpu_id TINYINT, \
-					entry_usec BIGINT,", grpid, hookid);
+				snprintf(sql, 1024, "create table %d_%d ( groupid TINYINT, hookid TINYINT, usec BIGINT, thread_id INT, process_id INT, parentprocess_id INT, cpu_id TINYINT, entry_usec BIGINT,", grpid, hookid);
 			} else  {
-				snprintf(sql, 1024, "create table %d_%d \
-					( groupid TINYINT, hookid TINYINT, \
-					usec BIGINT, process_id INT, \
-					cpu_id TINYINT,", grpid, hookid);
+				snprintf(sql, 1024, "create table %d_%d ( groupid TINYINT, hookid TINYINT, usec BIGINT, thread_id INT, process_id INT, parentprocess_id INT, cpu_id TINYINT,", grpid, hookid);
 			}
 		}
-		if(evt_type==2)  { /* if user event, alter an existing table */
+		if(evt_type==_HOOKID_REGUSREVT)  { /* if user event, alter an existing table */
 			snprintf(sql, 1024, "alter table %d_%d ", grpid, hookid);
 		}
 	}
@@ -726,21 +776,20 @@ void register_events(int evt_type, FILE *infp, size_t size)
 	name = strsep(&evt_names, ":");
 
 	if(fmt==NULL || name==NULL)  {
-		printf("error in event format/names string\n");
-		exit(-1);
+		b2a_error("error in event format/names string\n");
 	}
 
 	while(fmt!=NULL && name!=NULL)  {
 #ifdef HAS_MYSQL
 		if(into_db)  {
-			if(evt_type==1)  {
+			if(evt_type==_HOOKID_REGSYSEVT)  {
 				strcat(sql, "`");
 				strcat(sql, name);
 				strcat(sql, "` ");
 				strcat(sql, get_sqltype(fmt));
 				strcat(sql, ",");
 			}
-			if(evt_type==2)  {
+			if(evt_type==_HOOKID_REGUSREVT)  {
 				strcat(sql, "add ");
 				strcat(sql, "`");
 				strcat(sql, name);
@@ -765,15 +814,14 @@ void register_events(int evt_type, FILE *infp, size_t size)
 #ifdef HAS_MYSQL
 gen_sql:
 	if(into_db)  {
-		if(evt_type==1)
+		if(evt_type==_HOOKID_REGSYSEVT)
 			sql[strlen(sql)-1]=')';
-		if(evt_type==2)
+		if(evt_type==_HOOKID_REGUSREVT)
 			sql[strlen(sql)-1]='\0';
 
 		if(mysql_query(&mysql, sql))  {
-			fprintf(stderr, "Failed exec SQL: \n %s \n, Error: %s\n",
+			b2a_error("Failed exec SQL: \n %s \n, Error: %s\n",
 				sql, mysql_error(&mysql));
-			exit(-1);
 		}
 	}
 #endif
@@ -816,10 +864,10 @@ int dump_data(lket_pkt_header header, FILE *infp)
 
 	/* if the data contains user appended extra data */
 	if(header.total_size != header.sys_size)
-		evt_num = 2;
+		evt_num = 3;
 
 	/* iterate the sys and user event */
-	for(j=1; j<= evt_num; j++)  {
+	for(j=1; j<= evt_num; j+=2)  {
 		
 		readbytes = 0;
 
@@ -930,7 +978,7 @@ int dump_data(lket_pkt_header header, FILE *infp)
 					continue;
 				}
 				else {
-					return -1;
+					b2a_error("error processing STRING\n");
 				}
 			}
 		}
@@ -946,9 +994,8 @@ int dump_data(lket_pkt_header header, FILE *infp)
 
 		if(sql_count >= INSERT_THRESHOLD)  {
 			if(mysql_query(&mysql, sqlStatement))  {
-				fprintf(stderr, "Failed exec SQL:\n%s\n, Error:\n%s\n",
+				b2a_error("Failed exec SQL:\n%s\n, Error:\n%s\n",
 					sqlStatement, mysql_error(&mysql));
-				exit(-1);
 			}
 			while(!mysql_next_result(&mysql));
 			sql_count=0;
