@@ -1,6 +1,6 @@
 /* target operations
  * Copyright (C) 2005 Red Hat Inc.
- * Copyright (C) 2005,2006 Intel Corporation.
+ * Copyright (C) 2005, 2006, 2007 Intel Corporation.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -182,7 +182,7 @@
       case 1: __put_user_asm(((u8)(value)),addr,_bad,"b","b","iq",1); break;  \
       case 2: __put_user_asm(((u16)(value)),addr,_bad,"w","w","ir",1); break; \
       case 4: __put_user_asm(((u32)(value)),addr,_bad,"l","k","ir",1); break; \
-      case 8: __put_user_asm(((u64)(value)),addr,_bad,"q","","ir",1); break;  \
+      case 8: __put_user_asm(((u64)(value)),addr,_bad,"q","","Zr",1); break;  \
       default: __put_user_bad();					      \
       }									      \
     if (_bad)								      \
@@ -296,52 +296,24 @@
 
 #elif defined (__s390__) || defined (__s390x__)
 
-#if defined __s390__
-#define __stp_get_asm(x, addr, err, size)			\
-({								\
-	asm volatile(						\
-		"0: mvc  0(%2,%4),0(%3)\n"			\
-		"1:\n"						\
-		".section .fixup,\"ax\"\n"			\
-		"2: lhi    %0,%5\n"				\
-		"   bras    1,3f\n"				\
-		"   .long  1b\n"				\
-		"3: l      1,0(1)\n"				\
-		"   br     1\n"					\
-		".previous\n"					\
+#ifndef EX_TABLE
+/*
+ * Helper macro for exception table entries
+ */
+#ifndef __s390x__
+#define EX_TABLE(_fault,_target)				\
 		".section __ex_table,\"a\"\n"			\
-		"   .align 4\n"					\
-		"   .long  0b,2b\n"				\
-		".previous"					\
-		: "+&d" (err), "=m" (x)				\
-		: "i" (size),"a"(addr),				\
-		"a" (&(x)),"K" (-EFAULT)			\
-		: "cc", "1" );					\
-})
-
-#define __stp_put_asm(x, addr, err, size)			\
-({								\
-	asm volatile(						\
-		"0: mvc  0(%1,%2),0(%3)\n"			\
-		"1:\n"						\
-		".section .fixup,\"ax\"\n"			\
-		"2: lhi    %0,%5\n"				\
-		"   bras    1,3f\n"				\
-		"   .long  1b\n"				\
-		"3: l      1,0(1)\n"				\
-		"   br     1\n"					\
-		".previous\n"					\
+		"       .align 4\n"				\
+		"       .long  " #_fault "," #_target "\n"	\
+		".previous\n"
+#else
+#define EX_TABLE(_fault,_target)				\
 		".section __ex_table,\"a\"\n"			\
-		"   .align 4\n"					\
-		"   .long  0b,2b\n"				\
-		".previous"					\
-		: "+&d" (err)					\
-		: "i" (size), "a" (addr),			\
-		"a" (&(x)),"K" (-EFAULT)			\
-		: "cc", "1");					\
-})
-
-#else /* s390x */
+		"       .align 8\n"				\
+		"       .quad  " #_fault "," #_target "\n"	\
+		".previous\n"
+#endif
+#endif
 
 #define __stp_get_asm(x, addr, err, size)			\
 ({								\
@@ -352,35 +324,28 @@
 		"2: lghi    %0,%5\n"				\
 		"   jg     1b\n"				\
 		".previous\n"					\
-		".section __ex_table,\"a\"\n"			\
-		"   .align 8\n"					\
-		"   .quad  0b,2b\n"				\
-		".previous"					\
+		EX_TABLE(0b,2b)					\
 		: "+&d" (err), "=m" (x)				\
 		: "i" (size),"a"(addr),				\
 		"a" (&(x)),"K" (-EFAULT)			\
 		: "cc");					\
 })
 
-#define __stp_put_asm(x, addr, err, size)			\
+#define __stp_put_asm(x, addr, err)				\
 ({								\
 	asm volatile(						\
-		"0: mvc  0(%1,%2),0(%3)\n"			\
+		"0: stc %2,0(%1)\n"				\
 		"1:\n"						\
 		".section .fixup,\"ax\"\n"			\
-		"2: lghi    %0,%4\n"				\
+		"2: lghi    %0,%3\n"				\
 		"   jg     1b\n"				\
 		".previous\n"					\
-		".section __ex_table,\"a\"\n"			\
-		"   .align 8\n"					\
-		"   .quad  0b,2b\n"				\
-		".previous"					\
+		EX_TABLE(0b,2b)					\
 		: "+&d" (err)					\
-		: "i" (size),"a"(addr),				\
-		"a"(&(x)),"K"(-EFAULT)				\
+		: "a"(addr),					\
+		"r"(x),"K"(-EFAULT)				\
 		: "cc");					\
 })
-#endif
 
 #define deref(size, addr)					\
 ({								\
@@ -416,36 +381,19 @@
 	_v;							\
 })
 
-#define deref_store(size, addr, value)				\
-({								\
-	int _bad = 0;						\
-	switch (size) {						\
-	case 1:{						\
-		u8 _x = value;					\
-		__stp_put_asm(_x, addr, _bad,1);		\
-		break;						\
-		};						\
-	case 2:{ 						\
-		u16 _x = value;					\
-		__stp_put_asm(_x, addr, _bad,2);		\
-		break;						\
-		};						\
-	case 4:{						\
-		u32 _x = value;					\
-		__stp_put_asm(_x, addr, _bad,4);		\
-		break;						\
-		};						\
-	case 8:	{						\
-		u64 _x = value;					\
-		__stp_put_asm(_x, addr, _bad,8);		\
-		break;						\
-		};						\
-	default:						\
-		break;						\
-	}							\
-	if (_bad)						\
-		goto deref_fault;				\
+#define store_deref(size, addr, value)                          \
+({                                                              \
+        int _bad = 0;                                           \
+	int i;							\
+        for(i=0;i<size;i++){                                    \
+		__stp_put_asm((u8)(value>>((size-i-1)*8)&0xff),	\
+                             (u64)addr+i,_bad); 		\
+		if (_bad)					\
+			goto deref_fault;			\
+        }                                                       \
 })
+
+
 #endif /* (s390) || (s390x) */
 
 #define deref_string(dst, addr, maxbytes)				      \
@@ -461,3 +409,41 @@
     *_d = '\0';								      \
     (dst);								      \
   })
+
+
+#if defined __i386__
+
+/* x86 can't do 8-byte put/get_user_asm, so we have to split it */
+
+#define kread(ptr)					\
+  ((sizeof(*(ptr)) == 8) ?				\
+       *(typeof(ptr))&(u32[2]) {			\
+	 (u32) deref(4, &((u32 *)(ptr))[0]),		\
+	 (u32) deref(4, &((u32 *)(ptr))[1]) }		\
+     : (typeof(*(ptr))) deref(sizeof(*(ptr)), (ptr)))
+
+#define kwrite(ptr, value)						     \
+  ({									     \
+    if (sizeof(*(ptr)) == 8) {						     \
+      union { typeof(*(ptr)) v; u32 l[2]; } _kw;			     \
+      _kw.v = (typeof(*(ptr)))(value);					     \
+      store_deref(4, &((u32 *)(ptr))[0], _kw.l[0]);			     \
+      store_deref(4, &((u32 *)(ptr))[1], _kw.l[1]);			     \
+    } else								     \
+      store_deref(sizeof(*(ptr)), (ptr), (long)(typeof(*(ptr)))(value));     \
+  })
+
+#else
+
+#define kread(ptr) \
+  ( (typeof(*(ptr))) deref(sizeof(*(ptr)), (ptr)) )
+#define kwrite(ptr, value) \
+  ( store_deref(sizeof(*(ptr)), (ptr), (long)(typeof(*(ptr)))(value)) )
+
+#endif
+
+#define CATCH_DEREF_FAULT()				\
+  if (0) {						\
+deref_fault:						\
+    CONTEXT->last_error = "pointer dereference fault";	\
+  }
