@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * Map Functions
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005, 2006, 2007 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -39,10 +39,13 @@ int int64_eq_p (int64_t key1, int64_t key2)
 
 void str_copy(char *dest, char *src)
 {
-	int len = strlen(src);
-	if (len > MAP_STRING_LENGTH - 1)
-		len = MAP_STRING_LENGTH - 1;
-	memcpy (dest, src, len);
+	int len = 0;
+	if (src) {
+		len = strlen(src);
+		if (len > MAP_STRING_LENGTH - 1)
+			len = MAP_STRING_LENGTH - 1;
+		memcpy (dest, src, len);
+	}
 	dest[len] = 0;
 }
 
@@ -187,7 +190,7 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 		return -1;
 	}
 	if (max_entries) {
-		int i;
+		unsigned i;
 		void *tmp;
 		
 		/* size is the size of the map_node. */
@@ -204,12 +207,17 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 			if (cpu < 0)
 				tmp = kmalloc(size, STP_ALLOC_FLAGS);
 			else
-				tmp = kmalloc_node(size, STP_ALLOC_FLAGS, cpu);
+				tmp = kmalloc_node(size, STP_ALLOC_FLAGS, cpu_to_node(cpu));
 		
 			if (!tmp)
-				return -1;;
+				return -1;
 			
-			dbug ("allocated %lx\n", (long)tmp);
+			if (cpu < 0)
+				_stp_allocated_memory += size;
+			else
+				_stp_allocated_memory += size * num_online_cpus();
+				
+//			dbug ("allocated %lx\n", (long)tmp);
 			list_add((struct list_head *)tmp, &m->pool);
 			((struct map_node *)tmp)->map = m;
 		}
@@ -222,11 +230,10 @@ static int _stp_map_init(MAP m, unsigned max_entries, int type, int key_size, in
 
 static MAP _stp_map_new(unsigned max_entries, int type, int key_size, int data_size)
 {
-	MAP m = (MAP) kmalloc(sizeof(struct map_root), STP_ALLOC_FLAGS);
+	MAP m = (MAP) _stp_kzalloc(sizeof(struct map_root));
 	if (m == NULL)
 		return NULL;
 
-	memset (m, 0, sizeof(struct map_root));
 	INIT_LIST_HEAD(&m->pool);
 	INIT_LIST_HEAD(&m->head);
 	if (_stp_map_init(m, max_entries, type, key_size, data_size, -1)) {
@@ -241,14 +248,14 @@ static PMAP _stp_pmap_new(unsigned max_entries, int type, int key_size, int data
 	int i;
 	MAP map, m;
 
-	PMAP pmap = (PMAP) kmalloc(sizeof(struct pmap), STP_ALLOC_FLAGS);
+	PMAP pmap = (PMAP) _stp_kzalloc(sizeof(struct pmap));
 	if (pmap == NULL)
 		return NULL;
 
-	memset (pmap, 0, sizeof(struct pmap));
 	pmap->map = map = (MAP) alloc_percpu (struct map_root);
 	if (map == NULL) 
 		goto err;
+	_stp_allocated_memory += sizeof(struct map_root) * num_online_cpus();
 
 	/* initialize the memory lists first so if allocations fail */
         /* at some point, it is easy to clean up. */
@@ -636,7 +643,7 @@ static int print_keytype (char *fmt, int type, key_data *kd)
 	case STRING:
 		if (*fmt != 's')
 			return 1;
-		_stp_print_cstr (kd->strp);
+		_stp_print (kd->strp);
 		break;
 	case INT64:
 		if (*fmt == 'x')
@@ -668,7 +675,7 @@ static void print_valtype (MAP map, char *fmt, struct map_node *ptr)
 	switch (map->type) {
 	case STRING:
 		if (*fmt == 's')
-			_stp_print_cstr(_stp_get_str(ptr));
+			_stp_print (_stp_get_str(ptr));
 		break;
 	case INT64:
 	{
@@ -732,11 +739,11 @@ void _stp_map_printn (MAP map, int n, const char *fmt)
 			if (*f)
 				f++;
 		}
-		_stp_print_cstr ("\n");
+		_stp_print_char ('\n');
 		if (n && (--n <= 0))
 			break;
 	}
-	_stp_print_cstr ("\n");
+	_stp_print_char ('\n');
 	_stp_print_flush();
 }
 
@@ -974,12 +981,11 @@ static int _new_map_set_int64 (MAP map, struct map_node *n, int64_t val, int add
 	if (map == NULL || n == NULL)
 		return -2;
 
-	if (val || map->list) {
-		if (add)
-			*(int64_t *)((long)n + map->data_offset) += val;
-		else
-			*(int64_t *)((long)n + map->data_offset) = val;
-	}
+	if (add)
+		*(int64_t *)((long)n + map->data_offset) += val;
+	else
+		*(int64_t *)((long)n + map->data_offset) = val;
+
 	return 0;
 }
 
@@ -988,12 +994,11 @@ static int _new_map_set_str (MAP map, struct map_node *n, char *val, int add)
 	if (map == NULL ||  n == NULL)
 		return -2;
 
-	if ((val && *val)|| map->list) {
-		if (add)
-			str_add((void *)((long)n + map->data_offset), val);
-		else
-			str_copy((void *)((long)n + map->data_offset), val);
-	}
+	if (add)
+		str_add((void *)((long)n + map->data_offset), val);
+	else
+		str_copy((void *)((long)n + map->data_offset), val);
+
 	return 0;
 }
 
