@@ -32,6 +32,15 @@
 #define REG_SP(regs) regs->gpr[1]
 #define REG_LINK(regs) regs->link
 
+#elif defined (__s390__) || defined (__s390x__)
+#ifndef __s390x__
+#define PSW_ADDR_AMODE	0x80000000UL
+#else /* __s390x__ */
+#define PSW_ADDR_AMODE	0x0000000000000000UL
+#endif /* __s390x__ */
+#define REG_IP(regs)	(((regs)->psw.addr) | PSW_ADDR_AMODE)
+#define REG_SP(regs)	(regs)->gprs[15]
+
 #else
 #error "Unimplemented architecture"
 #endif
