@@ -1,6 +1,6 @@
 /* -*- linux-c -*-
  * Statistics Aggregation
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005, 2007 Red Hat Inc.
  * Copyright (C) 2006 Intel Corporation
  *
  * This file is part of systemtap, and is free software.  You can
@@ -96,7 +96,7 @@ Stat _stp_stat_init (int type, ...)
 		}
 		va_end (ap);
 	}
-	st = (Stat) kmalloc (sizeof(struct _Stat), STP_ALLOC_FLAGS);
+	st = (Stat) _stp_kmalloc (sizeof(struct _Stat));
 	if (st == NULL)
 		return NULL;
 	
@@ -115,7 +115,7 @@ Stat _stp_stat_init (int type, ...)
 	}
 #endif
 	
-	agg = (stat *)kmalloc (size, STP_ALLOC_FLAGS);
+	agg = (stat *)_stp_kmalloc(size);
 	if (agg == NULL)
 		goto exit2;
 
@@ -143,7 +143,7 @@ exit1:
 void _stp_stat_del (Stat st)
 {
 	if (st) {
-		_stp_free_percpu (st->sd);
+		free_percpu (st->sd);
 		kfree (st->agg);
 		kfree (st);
 	}
@@ -246,7 +246,7 @@ static void __stp_stat_print (char *fmt, Stat st, stat *sd, int cpu)
 		if (*f)
 			f++;
 	}
-	_stp_print_cstr ("\n");
+	_stp_print_char('\n');
 	_stp_print_flush();	
 }
 
