@@ -1,7 +1,7 @@
 /* -*- linux-c -*- 
  * symbols.c - stp symbol and module functions
  *
- * Copyright (C) Red Hat Inc, 2006
+ * Copyright (C) Red Hat Inc, 2006, 2007
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -47,7 +47,7 @@ static int _stp_sym_type_ok(int type)
 /* we need to store all the parts we are interested in */
 static unsigned _stp_get_sym_sizes(struct module *m, unsigned *dsize)
 {
-	int i;
+	unsigned int i;
 	unsigned num = 0, datasize = 0;
 	for (i=0; i < m->num_symtab; i++) {
 		char *str = (char *)(m->strtab + m->symtab[i].st_name);
@@ -63,12 +63,11 @@ static unsigned _stp_get_sym_sizes(struct module *m, unsigned *dsize)
 /* allocate space for a module and symbols */
 static struct _stp_module * _stp_alloc_module(unsigned num, unsigned datasize)
 {
-	struct _stp_module *mod = (struct _stp_module *)kmalloc(sizeof(struct _stp_module), STP_ALLOC_FLAGS);
+	struct _stp_module *mod = (struct _stp_module *)_stp_kzalloc(sizeof(struct _stp_module));
 	if (mod == NULL)
 		goto bad;
 
-	memset(mod, 0, sizeof(struct _stp_module));
-	mod->symbols = (struct _stp_symbol *)kmalloc(num * sizeof(struct _stp_symbol), STP_ALLOC_FLAGS);
+	mod->symbols = (struct _stp_symbol *)_stp_kmalloc(num * sizeof(struct _stp_symbol));
 	if (mod->symbols == NULL) {
 		mod->symbols = (struct _stp_symbol *)_stp_vmalloc(num * sizeof(struct _stp_symbol));
 		if (mod->symbols == NULL)
@@ -76,7 +75,7 @@ static struct _stp_module * _stp_alloc_module(unsigned num, unsigned datasize)
 		mod->allocated = 1;
 	}
 
-	mod->symbol_data = kmalloc(datasize, STP_ALLOC_FLAGS);
+	mod->symbol_data = _stp_kmalloc(datasize);
 	if (mod->symbol_data == NULL) {
 		mod->symbol_data = _stp_vmalloc(datasize);
 		if (mod->symbol_data == NULL)
@@ -87,12 +86,13 @@ static struct _stp_module * _stp_alloc_module(unsigned num, unsigned datasize)
 	return mod;
 
 bad:
-	if (mod)
-		kfree(mod);
-	if (mod->allocated && mod->symbols)
-		vfree(mod->symbols);
-	else
-		kfree(mod->symbols);
+	if (mod) {
+		if (mod->allocated && mod->symbols)
+			vfree(mod->symbols);
+		else
+			kfree(mod->symbols);
+		kfree(mod); 
+	}
 	return NULL;
 }
 
@@ -108,10 +108,13 @@ static void _stp_del_module(struct _stp_module *mod)
 {
 	int i, num;
 
-	kbug("deleting %s\n", mod->name);
+	// kbug("deleting %s\n", mod->name);
+
+	/* signal relocation code to clear its cache */
+	_stp_module_relocate((char *)-1, NULL, 0);
 
 	/* remove module from the arrays */
-	for (num = 1; num < _stp_num_modules; num++) {
+	for (num = 0; num < _stp_num_modules; num++) {
 		if (_stp_modules[num] == mod)
 			break;
 	}
@@ -121,7 +124,7 @@ static void _stp_del_module(struct _stp_module *mod)
 	for (i = num; i < _stp_num_modules-1; i++)
 		_stp_modules[i] = _stp_modules[i+1];
 
-	for (num = 1; num < _stp_num_modules; num++) {
+	for (num = 0; num < _stp_num_modules; num++) {
 		if (_stp_modules_by_addr[num] == mod)
 			break;
 	}
@@ -153,10 +156,10 @@ static void _stp_free_modules(void)
 	int i;
 	unsigned long flags;
 
-	STP_LOCK_MODULES;
+	/* This only happens when the systemtap module unloads */
+	/* so there is no need for locks. */
 	for (i = _stp_num_modules - 1; i >= 0; i--)
 		_stp_del_module(_stp_modules[i]);
-	STP_UNLOCK_MODULES;
 }
 
 static unsigned long _stp_kallsyms_lookup_name(const char *name);
@@ -164,25 +167,24 @@ static unsigned long _stp_kallsyms_lookup_name(const char *name);
 /* process the KERNEL symbols */
 static int _stp_do_symbols(const char __user *buf, int count)
 {
-	unsigned datasize, num;
-	int i;
+	unsigned i, datasize, num;
 	struct _stp_symbol *s;
 
 	switch (_stp_symbol_state) {
 	case 0:
 		if (count != 8) {
-			printk("unexpected systemtap error: _stp_do_symbols: count=%d\n", count);
+			errk(" _stp_do_symbols: count=%d\n", count);
 			return -EFAULT;
 		}		
 		if (get_user(num, (unsigned __user *)buf))
 			return -EFAULT;
 		if (get_user(datasize, (unsigned __user *)(buf+4)))
 			return -EFAULT;
-		kbug("num=%d datasize=%d\n", num, datasize);
+		// kbug("num=%d datasize=%d\n", num, datasize);
 
 		_stp_modules[0] = _stp_alloc_module(num, datasize);
 		if (_stp_modules[0] == NULL) {
-			printk("unexpected systemtap error: cannot allocate memory\n");
+			errk("cannot allocate memory\n");
 			return -EFAULT;
 		}
 		_stp_symbol_state = 1;
@@ -190,13 +192,13 @@ static int _stp_do_symbols(const char __user *buf, int count)
 	case 1:
 		if (copy_from_user ((char *)_stp_modules[0]->symbols, buf, count))
 			return -EFAULT;
-		kbug("got stap_symbols, count=%d\n", count);
+		// kbug("got stap_symbols, count=%d\n", count);
 		_stp_symbol_state = 2;
 		break;
 	case 2:
 		if (copy_from_user (_stp_modules[0]->symbol_data, buf, count))
 			return -EFAULT;
-		kbug("got symbol data, count=%d\n", count);
+		// kbug("got symbol data, count=%d\n", count);
 		_stp_num_modules = 1;
 
 		
@@ -204,11 +206,12 @@ static int _stp_do_symbols(const char __user *buf, int count)
 		for (i = 0; i < _stp_modules[0]->num_symbols; i++) 
 			s[i].symbol += (long)_stp_modules[0]->symbol_data;
 		_stp_symbol_state = 3;
+                /* NB: this mapping is used by kernel/_stext pseudo-relocations. */
 		_stp_modules[0]->text = _stp_kallsyms_lookup_name("_stext");
 		_stp_modules_by_addr[0] = _stp_modules[0];
 		break;
 	default:
-		printk("systemtap error: unexpected symbol data of size %d.\n", count);
+		errk("unexpected symbol data of size %d.\n", count);
 	}
 	return count;
 }
@@ -238,7 +241,7 @@ static void _stp_swap_symbol(void *x, void *y, int size)
 /* Create a new _stp_module and load the symbols */
 static struct _stp_module *_stp_load_module_symbols (struct _stp_module *imod)
 {
-	int i, num=0;
+	unsigned i, num=0;
 	struct module *m = (struct module *)imod->module;
 	struct _stp_module *mod = NULL;
 	char *dataptr;
@@ -252,7 +255,7 @@ static struct _stp_module *_stp_load_module_symbols (struct _stp_module *imod)
 		mod = _stp_alloc_module_from_module(m);
 		if (mod == NULL) {
 			module_put(m);
-			printk("Systemtap failed to allocate memory for module.\n");
+			errk("failed to allocate memory for module.\n");
 			return NULL;
 		}
 
@@ -288,7 +291,7 @@ static int _stp_module_exists(struct _stp_module *mod)
 {
 	int i, res;
 	unsigned long flags;
-	kbug("exists? %s\n", mod->name);
+	// kbug("exists? %s\n", mod->name);
 	STP_LOCK_MODULES;
 	for (i = 1; i < _stp_num_modules; i++) {
 		res = strcmp(_stp_modules[i]->name, mod->name);
@@ -308,7 +311,7 @@ static void _stp_ins_module(struct _stp_module *mod)
 	int i, num, res;
 	unsigned long flags;
 
-	kbug("insert %s\n", mod->name);
+	// kbug("insert %s\n", mod->name);
 
 	STP_LOCK_MODULES;
 
@@ -345,24 +348,24 @@ static void _stp_ins_module(struct _stp_module *mod)
 static int _stp_do_module(const char __user *buf, int count)
 {
 	struct _stp_module tmpmod, *mod;
-	int i;
+	unsigned i;
 
-	if (count < sizeof(tmpmod)) {
-		printk("_stp_do_modules: expected %d and got %d\n", (int)sizeof(tmpmod), count);
+	if (count < (int)sizeof(tmpmod)) {
+		errk("expected %d and got %d\n", (int)sizeof(tmpmod), count);
 		return -EFAULT;
 	}
 	if (copy_from_user ((char *)&tmpmod, buf, sizeof(tmpmod)))
 		return -EFAULT;
 
-	kbug("Got module %s, count=%d(0x%x)\n", tmpmod.name, count,count);
+	// kbug("Got module %s, count=%d(0x%x)\n", tmpmod.name, count,count);
 
 	if (_stp_module_exists(&tmpmod))
 		return count;
 
 	/* copy in section data */
-	tmpmod.sections = kmalloc(count - sizeof(tmpmod), STP_ALLOC_FLAGS);
+	tmpmod.sections = _stp_kmalloc(count - sizeof(tmpmod));
 	if (tmpmod.sections == NULL) {
-		printk("_stp_do_module: unable to allocate memory.\n");
+		errk("unable to allocate memory.\n");
 		return -EFAULT;
 	}
 	if (copy_from_user ((char *)tmpmod.sections, buf+sizeof(tmpmod), count-sizeof(tmpmod))) {
@@ -375,7 +378,7 @@ static int _stp_do_module(const char __user *buf, int count)
 				 + (long)((long)tmpmod.sections + tmpmod.num_sections * sizeof(struct _stp_symbol)));
 	}
 
-	#ifdef DEBUG
+	#ifdef DEBUG_SYMBOLS
 	for (i = 0; i < tmpmod.num_sections; i++)
 		printk("section %d (stored at %p): %s %lx\n", i, &tmpmod.sections[i], tmpmod.sections[i].symbol, tmpmod.sections[i].addr);
 	#endif
@@ -392,7 +395,7 @@ static int _stp_do_module(const char __user *buf, int count)
 	return count;
 }
 
-static int _stp_transport_send (int type, void *data, int len);
+static int _stp_ctl_send (int type, void *data, int len);
 
 static int _stp_module_load_notify(struct notifier_block * self, unsigned long val, void * data)
 {
@@ -404,10 +407,10 @@ static int _stp_module_load_notify(struct notifier_block * self, unsigned long v
 	case MODULE_STATE_COMING:
 		dbug("module %s loaded\n", mod->name);
 		strlcpy(rmod.name, mod->name, STP_MODULE_NAME_LEN);
-		_stp_transport_send(STP_MODULE, &rmod, sizeof(struct _stp_module));
+		_stp_ctl_send(STP_MODULE, &rmod, sizeof(struct _stp_module));
 		break;
 	default:
-		printk("module loaded? val=%ld\n", val);
+		errk("module loaded? val=%ld\n", val);
 	}
 #endif
 	return 0;
