@@ -58,12 +58,7 @@ struct unparser
   // struct context {
   //   unsigned errorcount;
   //   unsigned busy;
-  //   unsigned actioncount;
-  //   unsigned nesting;
-  //   union {
-  //     struct { .... } probe_NUM_locals;
-  //     struct { .... } function_NAME_locals;
-  //   } locals [MAXNESTING];
+  //   ...
   // } context [MAXCONCURRENCY];
 
   virtual void emit_global (vardecl* v) = 0;
@@ -85,7 +80,7 @@ struct unparser
   //   ....
   // }
 
-  virtual void emit_probe (derived_probe* v, unsigned i) = 0;
+  virtual void emit_probe (derived_probe* v) = 0;
   // void probe_NUMBER (struct context* c) {
   //   ... lifecycle
   //   ....
